@@ -211,7 +211,7 @@ class CID:
                       digest: Union[bytes, Tuple[Multihash, bytes]],
                      ) -> _CIDSubclass:
         # pylint: disable = too-many-arguments
-        instance: _CIDSubclass = super().__new__(CID_subclass)
+        instance: _CIDSubclass = object.__new__(CID_subclass)
         instance._base = base
         assert version in (0, 1)
         instance._version = cast(Literal[0, 1], version)
@@ -464,8 +464,8 @@ class CID:
         return f"CID({repr(mb)}, {v}, {repr(mc)}, {repr(d.hex())})"
 
     @property
-    def _as_tuple(self) -> Tuple[Type["CID"], Multibase, int, Multicodec, bytes]:
-        return (CID, self.base, self.version, self.codec, self.digest)
+    def _as_tuple(self) -> Tuple[Type["CID"], int, Multicodec, bytes]:
+        return (CID, self.version, self.codec, self.digest)
 
     def __hash__(self) -> int:
         return hash(self._as_tuple)
