@@ -21,13 +21,13 @@ from typing_validation import validate
 
 from bases import (base2, base16, base8, base10, base36, base58btc, base58flickr, base58ripple,
                    base32, base32hex, base32z, base64, base64url, base45,)
+from multiformats_config.multibase import load_multibase_table
 
 from multiformats.multibase import raw
 from multiformats.varint import BytesLike
 from .raw import RawEncoder, RawDecoder
 from .err import MultibaseKeyError, MultibaseValueError
 
-
 class Multibase:
     """
         Container class for a multibase encoding.
@@ -39,7 +39,7 @@ class Multibase:
 
         :param name: the multibase name
         :type name: :obj:`str`
-        :param code: the multibase code, as single-char string or ``0xYZ`` hex-string of a byte
+        :param code: the multibase code, as single-char string or ``0x...`` hex-string of a non-empty bytestring
         :type code: :obj:`str`
         :param status: the multibase status
         :type status: ``'draft'``, ``'candidate'`` or ``'default'``, *optional*
@@ -54,21 +54,23 @@ class Multibase:
 
     __slots__ = ("__weakref__", "_name", "_code", "_status", "_description")
 
-    def __init__(self, *,
-                 name: str,
-                 code: str,
-                 status: str = "draft",
-                 description: str = ""
-                ):
+    def __new__(cls,
+                name: str,
+                code: str,
+                status: str = "draft",
+                description: str = ""
+                ) -> "Multibase":
         for arg in (name, code, status, description):
             validate(arg, str)
         name = Multibase._validate_name(name)
         code = Multibase.validate_code(code)
         status = Multibase._validate_status(status)
-        self._name = name
-        self._code = code
-        self._status = status
-        self._description = description
+        instance = super().__new__(cls)
+        instance._name = name
+        instance._code = code
+        instance._status = status
+        instance._description = description
+        return instance
 
     @staticmethod
     def _validate_name(name: Optional[str]) -> str:
@@ -80,31 +82,31 @@ class Multibase:
 
     @staticmethod
     def validate_code(code: str) -> str:
-        """
+        r"""
             Validates a multibase code and transforms it to single-character format (if in hex format).
 
             Example usage:
 
             >>> Multibase.validate_code("0x00")
-            '\\x00'
+            '\x00'
             >>> Multibase.validate_code("hi")
             MultibaseValueError: Multibase codes must be single-character strings
             or the hex digits '0xYZ' of a single byte.
 
-            :param code: the multibase code, as single character or ``0xYZ`` hex-string of a single byte
+            :param code: the multibase code, as single character or ``0x...`` hex-string of a non-empty bytestring
             :type code: :obj:`str`
 
             :raises ValueError: if the code is invalid
 
         """
         validate(code, str)
-        if re.match(r"^0x[0-9a-zA-Z][0-9a-zA-Z]$", code):
+        if re.match(r"^0x([0-9a-zA-Z][0-9a-zA-Z])+$", code):
             ord_code = int(code, base=16)
+            if ord_code in range(0x20, 0x7F):
+                raise MultibaseValueError("Multibase codes in hex format cannot be printable ASCII characters.")
             code = chr(ord_code)
         elif len(code) != 1:
-            raise MultibaseValueError("Multibase codes must be single-character strings or the hex digits '0xYZ' of a single byte.")
-        if ord(code) not in range(0x00, 0x80):
-            raise MultibaseValueError("Multibase codes must be ASCII characters.")
+            raise MultibaseValueError("Multibase codes must be single-character strings or the hex digits '0x...' of a non-empty bytestring.")
         return code
 
     @staticmethod
@@ -127,7 +129,7 @@ class Multibase:
 
     @property
     def code_printable(self) -> str:
-        """
+        r"""
             Printable version of :meth:`Multibase.code`:
 
             - if the code is a single non-printable ASCII character, returns the hex string of its byte
@@ -135,9 +137,9 @@ class Multibase:
 
             Example usage:
 
-            >>> identity = multibase.get(code="\\x00")
+            >>> identity = multibase.get(code="\x00")
             >>> identity.code
-            '\\x00'
+            '\x00'
             >>> identity.code_printable
             '0x00'
 
@@ -145,7 +147,9 @@ class Multibase:
         code = self.code
         ord_code = ord(code)
         if ord_code not in range(0x20, 0x7F):
-            return "0x"+base16.encode(bytes([ord_code]))
+            ord_code_num_bytes = max(1, math.ceil(ord_code.bit_length()/8))
+            ord_code_bytes = ord_code.to_bytes(ord_code_num_bytes, byteorder="big")
+            return "0x"+base16.encode(ord_code_bytes)
         return code
 
     @property
@@ -526,35 +530,36 @@ def decode_raw(s: str) -> Tuple[Multibase, bytes]:
     base = from_str(s)
     return base, base.decode(s)
 
+_code_table, _name_table = load_multibase_table()
 
-def build_multibase_tables(bases: Iterable[Multibase]) -> Tuple[Dict[str, Multibase], Dict[str, Multibase]]:
-    """
-        Creates code->encoding and name->encoding mappings from a finite iterable of encodings, returning the mappings.
+# def build_multibase_tables(bases: Iterable[Multibase]) -> Tuple[Dict[str, Multibase], Dict[str, Multibase]]:
+#     """
+#         Creates code->encoding and name->encoding mappings from a finite iterable of encodings, returning the mappings.
 
-        Example usage:
+#         Example usage:
 
-        >>> code_table, name_table = build_multicodec_tables(bases)
+#         >>> code_table, name_table = build_multicodec_tables(bases)
 
-        :param bases: the multibases to add to the table
-        ::
+#         :param bases: the multibases to add to the table
+#         ::
 
-        :raises ValueError: if the same encoding code or name is encountered multiple times
-    """
-    # validate(multicodecs, Iterable[Multicodec]) # TODO: not yet properly supported by typing-validation
-    code_table: Dict[str, Multibase] = {}
-    name_table: Dict[str, Multibase] = {}
-    for e in bases:
-        if e.code in code_table:
-            raise MultibaseValueError(f"Multicodec name {e.name} appears multiple times in table.")
-        code_table[e.code] = e
-        if e.name in name_table:
-            raise MultibaseValueError(f"Multicodec name {e.name} appears multiple times in table.")
-        name_table[e.name] = e
-    return code_table, name_table
+#         :raises ValueError: if the same encoding code or name is encountered multiple times
+#     """
+#     # validate(multicodecs, Iterable[Multicodec]) # TODO: not yet properly supported by typing-validation
+#     code_table: Dict[str, Multibase] = {}
+#     name_table: Dict[str, Multibase] = {}
+#     for e in bases:
+#         if e.code in code_table:
+#             raise MultibaseValueError(f"Multicodec name {e.name} appears multiple times in table.")
+#         code_table[e.code] = e
+#         if e.name in name_table:
+#             raise MultibaseValueError(f"Multicodec name {e.name} appears multiple times in table.")
+#         name_table[e.name] = e
+#     return code_table, name_table
 
 # Create the global code->multibase and name->multibase mappings.
-_code_table: Dict[str, Multibase]
-_name_table: Dict[str, Multibase]
-with importlib_resources.open_text("multiformats.multibase", "multibase-table.json") as _table_f:
-    _table_json = json.load(_table_f)
-    _code_table, _name_table = build_multibase_tables(Multibase(**row) for row in _table_json)
+# _code_table: Dict[str, Multibase] = {}
+# _name_table: Dict[str, Multibase] = {}
+# with importlib_resources.open_text("multiformats.multibase", "multibase-table.json", encoding="utf8") as _table_f:
+#     _table_json = json.load(_table_f)
+#     _code_table, _name_table = build_multibase_tables(Multibase(**row) for row in _table_json)
